@@ -1,6 +1,6 @@
 /*
 htop
-(C) 2004-2010 Hisham H. Muhammad
+(C) 2004-2011 Hisham H. Muhammad
 Released under the GNU GPL, see the COPYING file
 in the source distribution for its full text.
 */
@@ -48,10 +48,4 @@ void Object_setClass(void* this, char* class) {
    ((Object*)this)->class = class;
 }
 
-static void Object_display(Object* this, RichString* out) {
-   char objAddress[50];
-   sprintf(objAddress, "%s @ %p", this->class, (void*) this);
-   RichString_write(out, CRT_colors[DEFAULT_COLOR], objAddress);
-}
-
 #endif
